//
//  The MENU CLASS
//
// isDisplayable() - return boolean whether this menu should be displayed.
// getTitle() - return the title or label for this menu.
// getImage() - return the image for this menu.
// getItems() - returns the array of MenuItems or Menus which this menu contains.
//
function Menu(title,raw,notype)
{
    this.title = title;
    this.raw = raw;
    this.img = "";
    this.type = "";
    this.items = new Array();
    this.offset = 0;
    this.parse(this.raw, notype);
}

Menu.prototype.isDisplayable = function()
{
    if (this.type == "alert")
    {
        if(this.items.length == 0)
            return false;
    }
    return true;
}

Menu.prototype.getType = function()
{
    return this.type;
}

Menu.prototype.getTitle = function()
{
    // on IE, the Toolbar Settings has no text, but
    // I can't find how to do that here...
    if (this.type == "option" || this.type == "alert" || this.type == "logo")
        return "";
    else
        return this.title;
}

Menu.prototype.getTooltip = function()
{
   return this.title;
}

Menu.prototype.getImage = function()
{
    return this.img;
}

Menu.prototype.getItems = function()
{
    return this.items;
}

Menu.prototype.parse = function(txt, notype)
{
    var end;
    if(notype != null)
    {
        //        alert("Not trying to find menu type");
        end = 0;
    }
    else
    {
        end = txt.indexOf('\x18');
        this.img = txt.substr(0, end);
        if (this.img == "mno2.bmp")
        	this.type = "alert";
        else
        	this.type = this.img;
    }
    var menuLineItems = txt.substr(end);
    //var testEol = txt.indexOf('\x19');
    //var mlItems2 = txt.substr(testEol);
    //alert("Type is '" + this.type + "' and menuLineItems: " + menuLineItems +'\n'
          //+ ' EOL: ' + testEol + '\n'
          //+ ' mlItems2: ' + mlItems2
          //);
    this.parseLineItems(menuLineItems);
    this.length = end + this.offset;
    
}


Menu.prototype.parseLineItems = function(txt)
{
    // find the first ^V, that's the start of the menu
    // then ^V to ^Y^W is one item in the menu
    this.offset = txt.indexOf('\x16', this.offset);
    // ^A = \x01
    // ^B = \x02
    // ^C = \x03
    // ^V = \x16 (MO)
    // ^W = \x17 (MC)
    // ^X = \x18 (NV)
    // ^Y = \x19 (EOL)

    // this algorithm works if there are no nested menus.
    // a simple menu looks like:
    // ^V .... ^W
    // ^V .... ^W
    //
    // but the problem is that a menu line item with a nested menu looks like:
    // ^V <title> ^X ^C ^Y ^V <subtitle> ^X ... <link>^Y ^W
    // ...
    // ^Y^V <subtitle> ^X <link> ^Y ^W
    // ^W
    // so, here, we need to determine if string represents a simple line item
    // or if its a nested menu.
    do {
        if (txt.charAt(this.offset) == '\x14')    // Iconic menuitem
	{
	   var iconic = true;
           var startMenuItem = this.offset + 4;   // Skip ^TY!^X
	   var endMenuItem = txt.indexOf('\x15', this.offset);
	} else {
	   var iconic = false;
           var startMenuItem = txt.indexOf('\x16', this.offset) + 1;
	   var endMenuItem = txt.indexOf('\x17', startMenuItem);
	}
	if (startMenuItem < 1 || endMenuItem < 1) break;

        // fetch the title... up to the first ^X
        var endTitle = txt.indexOf('\x18', startMenuItem);
        var title = txt.substr((startMenuItem), (endTitle - (startMenuItem)));
        
	// The menuline is not used for nested menus
        var menuLine = txt.substr((startMenuItem), (endMenuItem - (startMenuItem)));

        // now we need to examine the character after the ^X
        // if it's a ^C, we have a nested menu,
        var idChar = txt.charAt((endTitle + 1));
	if (iconic)
	{
	  var items = menuLine.split('\x18');        // Separate elements
	  idChar = '\x01';
	  menuLine  = items[0] + '\x18' + items[3] + '\x18';

        }
 
        if(idChar == '\x03')
        {
            //var nestedMenuString = null;
            //nestedMenuString = this.findNestedMenu(txt, (endTitle+3));
            //this.items.push(new Menu(title, nestedMenuString, 1));
            var nMenu = new Menu(title, txt.substr(endTitle+3), 1);
            this.items.push(nMenu);
            this.offset = endTitle + 3 + nMenu.length +1;
        } else {
            if(endMenuItem == -1)
            {
                // this always happens in a nested menu... but it's a good
                // sanity check anyway.
                //alert("HEY! couldn't find a ^Y to end this!");
                break;
            }
            this.items.push(new MenuItem(menuLine)); 
            this.offset = endMenuItem + 1;
        }
        if (txt.charAt(this.offset) == '\x17')
        	return;
    } while(this.offset < txt.length);
    //alert("Done parsing menu named " + this.title);
}

Menu.prototype.dumpItems = function(indent)
{
    var msg = "Menu title: " + this.title + "\n";
    var len = this.items.length;
    var i=0;
    for( ; i < len ; i++)
    {
        var mli = this.items[i];
        if(indent != null)
            msg += indent;
        if(mli.constructor == MenuItem)
        {
            
            msg += "Txt: " + mli.menuText + " Link: " + mli.menuLink + "\n";
        }
        else
        {
            // this is a nested menu!
            msg += "NESTED: \n";
            var newindent = (indent != null)? indent+"\t" : "\t";
            msg += mli.dumpItems(newindent);
            if(indent != null) msg += indent;
            msg+= "END NESTED\n";
        }
    }
    return msg;
}
Menu.prototype.getTxt = function()
{
	var retVal;
    var len = this.items.length;
    var i=0;
    
	for( ; i < len ; i++)
    {
        var mli = this.items[i];
        if(mli.constructor == MenuItem)
        {
            retVal = mli.menuText;
        }
    }
    return retVal;
}
Menu.prototype.htmlDumpItems = function(indent)
{
    var msg = "<br><b>'" + this.title + "'</b> \n";
    var len = this.items.length;
    var i=0;
    for( ; i < len ; i++)
    {
        var mli = this.items[i];
        if(indent != null)
            msg += indent;

        if(mli.constructor == MenuItem)
        {
            if (mli.menuText != "-") {
                msg += "<br>Txt: '" + mli.menuText + "' Link: '" + mli.menuLink + "'\n";
            }
        }
        else
        {
            // this is a nested menu!
            msg += "<div style=\"margin-left: 40px;\"> \n";
            var newindent = (indent != null) ? indent + "\t" : "\t";
            msg += mli.dumpItems(newindent);
            
            if (indent != null) 
                msg += indent;
            msg+= "</div> \n";
        }
    }
    return msg;
}
Menu.prototype.breakLine = function(line)
{
      var tmp1 = "";
      var tmp2 = "";
      var start = 30;
      while (start < line.length) {
        tmp1 = line.substring(0, start);
        tmp2 = line.substring(start, line.length);
        line = tmp1 + "\n" + tmp2;
        start += 30;
      }
      return line;
}

